%% SRDM Approach (Sensitivity Analysis on the Number of Decision Points)
%  This code is the modified version of Sections 5 and 6 of Modified_LSM_Algorithm.m
%  It determines the optimal decision policy by applying different number of decision points as defined. 

% 1. Path Table Generation
N_MC = size(ObsRate_Phase1n2_real_time_ShiftTime,1);
FieldLifeTime = size(ObsRate_Phase1n2_real_time_ShiftTime,2);
SRDM_Shift_Stop_OptNPV_real = zeros(N_MC,1);
Shift_time = fliplr(LT_DecisionPoint);

% Determining the optimal stopping time for each decision point (switch time)
PathTable = zeros(N_MC,N_DecisionPoint+1); 
PathTable(:,end) = NPVtable_LTPhase1_LTPhase2_real_DiffDP(N_DecisionPoint+1,1,:); 
Phase2_StopTime_real_Phase1LifeTime_DiffDP = zeros(N_MC,N_DecisionPoint+1); 

for k_DecisionPoint = 2: N_DecisionPoint+1 
    
        NPV_2Phases_matrix = zeros(N_MC,k_DecisionPoint); 
        
    for k_Stop_time = 1: k_DecisionPoint           
        NPV_2Phases_matrix(:,k_Stop_time) = NPVtable_LTPhase1_LTPhase2_real_DiffDP(N_DecisionPoint-k_DecisionPoint+2,k_Stop_time,:);
    end

    for k_StopTime = 1: k_DecisionPoint-1     
        if k_StopTime == 1 
            x_data_stop_SRDM = ObsRate_Phase1n2_real_time_ShiftTime_DiffDP(:,1:Shift_time(k_StopTime+1),Shift_time(k_DecisionPoint)+1); %%HERE 
            X_stop_SRDM = [ones(N_MC,1) x_data_stop_SRDM]; 
            y_data_stop_matrix = [NPV_2Phases_matrix(:,k_DecisionPoint-k_StopTime) NPV_2Phases_matrix(:,k_DecisionPoint)];
            [Stop_value,Stop_idx] = max([X_stop_SRDM*regress(NPV_2Phases_matrix(:,k_DecisionPoint-k_StopTime),X_stop_SRDM) X_stop_SRDM*regress(NPV_2Phases_matrix(:,k_DecisionPoint),X_stop_SRDM)],[],2);
            for i_MC = 1: N_MC 
                SRDM_Shift_Stop_OptNPV_real(i_MC,1) = y_data_stop_matrix(i_MC,Stop_idx(i_MC));
            end    
        else
            x_data_stop_SRDM = ObsRate_Phase1n2_real_time_ShiftTime_DiffDP(:,1:Shift_time(k_StopTime+1),Shift_time(k_DecisionPoint)+1); %%HERE 
            X_stop_SRDM = [ones(N_MC,1) x_data_stop_SRDM]; 
            y_data_stop_matrix = [NPV_2Phases_matrix(:,k_DecisionPoint-k_StopTime) SRDM_Shift_Stop_OptNPV_real(:,1)];
            [Stop_value,Stop_idx] = max([X_stop_SRDM*regress(NPV_2Phases_matrix(:,k_DecisionPoint-k_StopTime),X_stop_SRDM) X_stop_SRDM*regress(SRDM_Shift_Stop_OptNPV_real(:,1),X_stop_SRDM)],[],2);
            for i_MC = 1: N_MC
                SRDM_Shift_Stop_OptNPV_real(i_MC,1) = y_data_stop_matrix(i_MC,Stop_idx(i_MC));
            end  
        end
    end     
  
    for i_MC = 1: N_MC 
        PathTable(i_MC,N_DecisionPoint-k_DecisionPoint+2) = SRDM_Shift_Stop_OptNPV_real(i_MC,1);
        for k_Stop_time = 1: k_DecisionPoint  
            if SRDM_Shift_Stop_OptNPV_real(i_MC,1) == NPV_2Phases_matrix(i_MC,k_Stop_time)
                Phase2_StopTime_real_Phase1LifeTime_DiffDP(i_MC,N_DecisionPoint-k_DecisionPoint+2) = LT_DecisionPoint(k_Stop_time);
            end
        end
    end  
end

% 2. SRDM Approach (Modified LSM Algorithm)

y_data_continuation = zeros(N_MC,1);
y_data_shift = zeros(N_MC,1);
y_reg_continuation = zeros(N_MC,1);
y_reg_shift = zeros(N_MC,1);

% Determining the optimal switch time 
ValueTable = zeros(N_MC,N_DecisionPoint+1);
ValueTable(:,end) = PathTable(:,end);
SRDM_DecisionTable = zeros(N_MC,N_DecisionPoint+1);
SRDM_DecisionTable(:,end) = 1;
x_data = ObsRate_Phase1n2_real_time_ShiftTime_DiffDP(:,:,LT_DecisionPoint(N_DecisionPoint+1)+1);

for k_Shift_time = 2: N_DecisionPoint+1 
    X = [ones(N_MC,1) x_data(:,1:Shift_time(k_Shift_time))]; 
    y_data_shift(:) = PathTable(:,N_DecisionPoint-k_Shift_time+2);  
    coef_shift = regress(y_data_shift,X);
    y_reg_shift(:) = X*coef_shift;
    y_data_continuation(:) = ValueTable(:,N_DecisionPoint-k_Shift_time+3); 
    coef_continuation = regress(y_data_continuation,X);
    y_reg_continuation(:) = X*coef_continuation;
    ENPVTable_Comparison = [y_reg_shift, y_reg_continuation];
    NPV_SRDM = [y_data_shift, y_data_continuation];
    [ENPV_SRDM,ENPV_SRDM_idx] = max(ENPVTable_Comparison,[],2);
    
    for i_MC = 1: N_MC
        ValueTable(i_MC,N_DecisionPoint-k_Shift_time+2) = NPV_SRDM(i_MC,ENPV_SRDM_idx(i_MC)); 
        if y_reg_continuation(i_MC) > y_reg_shift(i_MC)
            SRDM_DecisionTable(i_MC,N_DecisionPoint-k_Shift_time+2) = 0; 
        else
            SRDM_DecisionTable(i_MC,N_DecisionPoint-k_Shift_time+2) = 1; 
        end
    end
end

ShiftValue = PathTable(:,1);
ExpShiftValue = mean(ShiftValue);

ContinuationValue = ValueTable(:,1);
ExpContinuationValue = mean(ContinuationValue);
EVWII_DiffDP = ExpContinuationValue;
VOI_DiffDP = EVWII_DiffDP - EVWOI_DiffDP;
VOI_over_EVWOI_pct = VOI_DiffDP/EVWOI_DiffDP*100;

[~,DWI_Phase1LifeTime_DiffDP_idx] = max(SRDM_DecisionTable,[],2); 

DWI_Phase1LifeTime_DiffDP = LT_DecisionPoint(DWI_Phase1LifeTime_DiffDP_idx);
DWI_Phase1LifeTime_DiffDP = transpose(DWI_Phase1LifeTime_DiffDP);

DWI_Phase2LifeTime_DiffDP = zeros(N_MC,1);

for i_MC = 1:N_MC
    Phase1LifeTime_idx_real = DWI_Phase1LifeTime_DiffDP_idx(i_MC,1);
    DWI_Phase2LifeTime_DiffDP(i_MC,1) = Phase2_StopTime_real_Phase1LifeTime_DiffDP(i_MC,Phase1LifeTime_idx_real);
end

DWI_LifeTime_DiffDP = DWI_Phase1LifeTime_DiffDP+DWI_Phase2LifeTime_DiffDP;


