%% Mean Reverting Process (Ornstein-Uhlenbeck Process)
%  This code performs the stochastic modeling of the economic parameters
%  based on the OU model

N_MC = size(ObsRate_Phase1n2_real_time_ShiftTime,1);
FieldLifeTime = size(ObsRate_Phase1n2_real_time_ShiftTime,2);

% 1. OU Parameters for Oil Price - Inputs here are from calibration done by using least-squares approach
OilPrice = 47.48; 
theta_oilprice = 0.0643; %speed of mean reversion 
mu_oilprice = 47.48; %reverting mean 
sigma_oilprice = 0.2473; %process volatility %percent 
dt_oilprice = 1; %year
Pi_OilPrice_Matrix = zeros(N_MC,FieldLifeTime);
OilPrice_Matrix = zeros(N_MC,FieldLifeTime);

% 2. OU Parameters for Cost Multiplier 
% The parameters here are from Willigers (2008)
% These same parameters are used for Opex_P1, Opex_P2 & Capex_2After1
Initial_cost_multiplier = 1;
theta_cost_multiplier = 0.1; %speed of mean reversion 
mu_cost_multiplier = 1; %reverting mean
sigma_cost_multiplier = 0.5; %process volatility %percent
dt_cost_multiplier = 1; %year
Cost_Multiplier_Opex_Phase1 = zeros(N_MC,FieldLifeTime);
Cost_Multiplier_Opex_Phase2 = zeros(N_MC,FieldLifeTime);
Cost_Multiplier_Capex_Phase2After1 = zeros(N_MC,FieldLifeTime);
Pi_Opex_Phase1_Matrix = zeros(N_MC,FieldLifeTime);
Pi_Opex_Phase2_Matrix = zeros(N_MC,FieldLifeTime);
Pi_Capex_Phase2After1_Matrix = zeros(N_MC,FieldLifeTime);

% 3. Generating random samples using multivariate normal distibution 

mvnd_mean = [0,0,0,0];
mvnd_SD = [1,1,1,1];
exp_CorrC = [1 0.87 0.87 0.85; 0.87 1 0.55 0.65; 0.87 0.55 1 0.7; 0.85 0.65 0.7 1];
% There are 4 by 4 Correlation Coefficient Matrix used in generating the multivariate uniform distribution 
% Since no data, except for oil price, is available, these coefficients are mainly assumed
cov = exp_CorrC.*(mvnd_SD'*mvnd_SD);

mvnd_samples = zeros(N_MC,4,FieldLifeTime);
reshaped_mvnd_samples = zeros(N_MC,FieldLifeTime,4);

for t = 1: FieldLifeTime %these loops are to generate 4 different types samples from mvnrnd for each year throughout the FieldLifeTime
    mvnd_samples(:,:,t) = mvnrnd(mvnd_mean,cov,N_MC); 
    reshaped_mvnd_samples(:,t,:) = mvnd_samples(:,:,t);
end


% 4. Generation of the Values of Economic Parameters
 
for t = 1: FieldLifeTime %the reshaped_mnvd_samples are used in the OU model 
    if t == 1
        Pi_OilPrice_Matrix(:,t) = (log(OilPrice)*exp(-theta_oilprice*dt_oilprice)) + (log(mu_oilprice)*(1-exp(-theta_oilprice*dt_oilprice))) + (sigma_oilprice*sqrt((1-exp(-2*theta_oilprice*dt_oilprice))/(2*theta_oilprice))*reshaped_mvnd_samples(:,t,1));
        OilPrice_Matrix(:,t) = exp(Pi_OilPrice_Matrix(:,t)-(0.5*(sigma_oilprice^2/(2*theta_oilprice))*(1-exp(-2*theta_oilprice*t))));
        Pi_Opex_Phase1_Matrix(:,t) = (log(Initial_cost_multiplier)*exp(-theta_cost_multiplier*dt_cost_multiplier)) + (log(mu_cost_multiplier)*(1-exp(-theta_cost_multiplier*dt_cost_multiplier))) + (sigma_cost_multiplier*sqrt((1-exp(-2*theta_cost_multiplier*dt_cost_multiplier))/(2*theta_cost_multiplier))*reshaped_mvnd_samples(:,t,2));
        Cost_Multiplier_Opex_Phase1(:,t) = exp(Pi_Opex_Phase1_Matrix(:,t)-(0.5*(sigma_cost_multiplier^2/(2*theta_cost_multiplier))*(1-exp(-2*theta_cost_multiplier*t))));
        Pi_Opex_Phase2_Matrix(:,t) = (log(Initial_cost_multiplier)*exp(-theta_cost_multiplier*dt_cost_multiplier)) + (log(mu_cost_multiplier)*(1-exp(-theta_cost_multiplier*dt_cost_multiplier))) + (sigma_cost_multiplier*sqrt((1-exp(-2*theta_cost_multiplier*dt_cost_multiplier))/(2*theta_cost_multiplier))*reshaped_mvnd_samples(:,t,3));
        Cost_Multiplier_Opex_Phase2(:,t) = exp(Pi_Opex_Phase2_Matrix(:,t)-(0.5*(sigma_cost_multiplier^2/(2*theta_cost_multiplier))*(1-exp(-2*theta_cost_multiplier*t))));
        Pi_Capex_Phase2After1_Matrix(:,t) = (log(Initial_cost_multiplier)*exp(-theta_cost_multiplier*dt_cost_multiplier)) + (log(mu_cost_multiplier)*(1-exp(-theta_cost_multiplier*dt_cost_multiplier))) + (sigma_cost_multiplier*sqrt((1-exp(-2*theta_cost_multiplier*dt_cost_multiplier))/(2*theta_cost_multiplier))*reshaped_mvnd_samples(:,t,4));
        Cost_Multiplier_Capex_Phase2After1(:,t) = exp(Pi_Capex_Phase2After1_Matrix(:,t)-(0.5*(sigma_cost_multiplier^2/(2*theta_cost_multiplier))*(1-exp(-2*theta_cost_multiplier*t))));
    else
        Pi_OilPrice_Matrix(:,t) = (Pi_OilPrice_Matrix(:,t-1)*exp(-theta_oilprice*dt_oilprice)) + (log(mu_oilprice)*(1-exp(-theta_oilprice*dt_oilprice))) + (sigma_oilprice*sqrt((1-exp(-2*theta_oilprice*dt_oilprice))/(2*theta_oilprice))*reshaped_mvnd_samples(:,t,1));
        OilPrice_Matrix(:,t) = exp(Pi_OilPrice_Matrix(:,t)-(0.5*(sigma_oilprice^2/(2*theta_oilprice))*(1-exp(-2*theta_oilprice*t))));
        Pi_Opex_Phase1_Matrix(:,t) = (Pi_Opex_Phase1_Matrix(:,t-1)*exp(-theta_cost_multiplier*dt_cost_multiplier)) + (log(mu_cost_multiplier)*(1-exp(-theta_cost_multiplier*dt_cost_multiplier))) + (sigma_cost_multiplier*sqrt((1-exp(-2*theta_cost_multiplier*dt_cost_multiplier))/(2*theta_cost_multiplier))*reshaped_mvnd_samples(:,t,2));
        Cost_Multiplier_Opex_Phase1(:,t) = exp(Pi_Opex_Phase1_Matrix(:,t)-(0.5*(sigma_cost_multiplier^2/(2*theta_cost_multiplier))*(1-exp(-2*theta_cost_multiplier*t))));
        Pi_Opex_Phase2_Matrix(:,t) = (Pi_Opex_Phase2_Matrix(:,t-1)*exp(-theta_cost_multiplier*dt_cost_multiplier)) + (log(mu_cost_multiplier)*(1-exp(-theta_cost_multiplier*dt_cost_multiplier))) + (sigma_cost_multiplier*sqrt((1-exp(-2*theta_cost_multiplier*dt_cost_multiplier))/(2*theta_cost_multiplier))*reshaped_mvnd_samples(:,t,3));
        Cost_Multiplier_Opex_Phase2(:,t) = exp(Pi_Opex_Phase2_Matrix(:,t)-(0.5*(sigma_cost_multiplier^2/(2*theta_cost_multiplier))*(1-exp(-2*theta_cost_multiplier*t))));
        Pi_Capex_Phase2After1_Matrix(:,t) = (Pi_Capex_Phase2After1_Matrix(:,t-1)*exp(-theta_cost_multiplier*dt_cost_multiplier)) + (log(mu_cost_multiplier)*(1-exp(-theta_cost_multiplier*dt_cost_multiplier))) + (sigma_cost_multiplier*sqrt((1-exp(-2*theta_cost_multiplier*dt_cost_multiplier))/(2*theta_cost_multiplier))*reshaped_mvnd_samples(:,t,4));
        Cost_Multiplier_Capex_Phase2After1(:,t) = exp(Pi_Capex_Phase2After1_Matrix(:,t)-(0.5*(sigma_cost_multiplier^2/(2*theta_cost_multiplier))*(1-exp(-2*theta_cost_multiplier*t))));
    end
end

Opex_Phase1_Matrix = zeros(N_MC,FieldLifeTime);
Opex_Phase2_Matrix = zeros(N_MC,FieldLifeTime);
Capex_Phase2After1_Matrix = zeros(N_MC,FieldLifeTime);

for t = 1: FieldLifeTime
    if t == 1 %for Costs at Y1, Oil Price at Year 0 is used 
        Opex_Phase1_Matrix(:,t) = OilPrice.*Cost_Multiplier_Opex_Phase1(:,t);
        Opex_Phase2_Matrix(:,t) = OilPrice.*Cost_Multiplier_Opex_Phase2(:,t);
        Capex_Phase2After1_Matrix(:,t) = OilPrice.*Cost_Multiplier_Capex_Phase2After1(:,t);
    else %for Costs at Year n+1, Oil Price at Year n is used 
        Opex_Phase1_Matrix(:,t) = OilPrice_Matrix(:,t-1).*Cost_Multiplier_Opex_Phase1(:,t);
        Opex_Phase2_Matrix(:,t) = OilPrice_Matrix(:,t-1).*Cost_Multiplier_Opex_Phase2(:,t);
        Capex_Phase2After1_Matrix(:,t) = OilPrice_Matrix(:,t-1).*Cost_Multiplier_Capex_Phase2After1(:,t);   
    end
end
